gsap.registerPlugin(MotionPathPlugin,ScrollTrigger, ScrollToPlugin);

/**
 * JrWebDesign is a global object designed to handle all tasks across the website.
 * It provides utility methods and functionality that can be accessed and reused
 * throughout the project, ensuring consistency and reducing code duplication.
 * 
 * @example
 * JrWebDesign.preload(); // Call to start the preloading process.
 * JrWebDesign.header(); // Call to Initialize header functionality globally 
 */
let JrWebDesign = {
  // Settings for animations
  toggleAnimations:true,
  defaults:{
    duration:1.5,
    ease:"expo.out"
  },
  /**
   * This Method designed for preload task
   * @type Global
  */
  preload($scope, json = '', onComplete = ()=>{}){  
    var preload = $scope.querySelector('.jr__preload');
    // var json = preload.getAttribute('data-path');
    var lottieAni = lottie.loadAnimation({
        container:preload,
        path: json,
        autoplay: false,
        renderer: 'svg', 
        loop: false,
        progressiveLoad: true,
        rendererSettings: {
            preserveAspectRatio: 'xMidYMid meet',
            progressiveLoad: true, 
        }
    })
    lottieAni.setSubframe(false);
    lottieAni.setSpeed(1);
    lottieAni.addEventListener("DOMLoaded", ()=>{
      var fG = preload.querySelector("svg > g");
      fG.setAttribute('clip-path', '');
      lottieAni.play();
    })
    lottieAni.addEventListener("complete", ()=>{
      gsap.to(preload, {
        scale:5,
        duration:.2,
        onComplete(){
          gsap.to($scope, {  
            autoAlpha:0,
            duration:.5,
            onComplete(){
              $scope.classList.add('d-none');
              onComplete();
            }
          })
        }
      })
    });
  },
  /**
     * Utility method to fix animation starting issues.
     * This method hides the specified elements initially to prevent animation glitches.
     * After the animation setup, call the `done()` method to reset the visibility.
     * @example
     * const handler = JrWebDesign.beforeAni('.animated-elements');
     * // Perform animation setup or other tasks here
     * handler.done(); // Restore visibility after tasks are completed.
  */
  beforeAni(elements){ 
    gsap.set(elements, {'visibility':'hidden'});    
    return{
      done(){
        gsap.set(elements, {visibility:''});
      }
    }
  },
  header($scope){
    var _this = this;
    var toggleBtn = $scope.querySelector('.header-btn-toggle');
    var mobileMenuWrap = $scope.querySelector('.mobile-menu-wrapper');
    // Open Menu
    let fadeItem = [".mobile-menu-content .sm-menu > .menu-item", ".mobile-menu-wrapper .fadeIn"]; 
    let topShape = $scope.querySelector(".mobile-menu-wrapper .top-shape");
    const openMenu = ()=>{ 
      let MHeight = mobileMenuWrap.classList.contains('menu-y-full') ? "100vh" : "auto";
      // SubMenus.each(function(){$(this).closest('.menu-item').css('--lh',$(this).closest('.menu-item').find("> a").innerHeight()+"px")});
      // SubMenus.each(function(){$(this).closest('.menu-item').css('--lw',$(this).closest('.menu-item').find("> a").innerWidth()+"px")});  
      gsap.set(fadeItem, {
        opacity:0,
        y:20 
      })
      gsap.set(topShape, {
        y:-50
      })
      gsap.to(mobileMenuWrap, {
        'height':MHeight,
        ease:'expo.out',
        duration:.6
      });
      gsap.to(fadeItem, {
        opacity:1,
        y:0,
        ease:"expo.out", 
        duration:1,
        stagger:0.1
      })
      gsap.to(topShape, {
        y:0,
        ease:"expo.out", 
        duration:.5,
      })
    }
    // Close Menu
    const openClose = ()=>{ 
      gsap.to(mobileMenuWrap, {
        'height':0,
        ease:'expo.out'
      });
      // gsap.to(fadeItem, {
      //   opacity:0,
      //   y:20
      // })
    }
    this.click(toggleBtn, ()=>{
      toggleBtn.classList.toggle('open');
      if(toggleBtn.classList.contains('open')){
        openMenu();
      }else{
        openClose();
      }
    });
    let lastScroll = 0; 
    window.addEventListener('scroll', (e) => {
        const currentScroll = window.scrollY || document.documentElement.scrollTop;
        if (currentScroll > lastScroll) {
            document.querySelector('body').classList.add('scroll-down');
            document.querySelector('body').classList.remove('scroll-up');
        } else {
          document.querySelector('body').classList.add('scroll-up');
          document.querySelector('body').classList.remove('scroll-down');
        }
        lastScroll = currentScroll <= 0 ? 0 : currentScroll;
        if(window.scrollY > 50){
          $scope.classList.add('active');
        }else{
          $scope.classList.remove('active');
        }
    });
  },
  smoothScroll(){
    let lenis = null;
    if(window.innerWidth > 991){
      lenis = new Lenis({
        duration:.75,
      });
      lenis.on('scroll', ()=>{ 
        ScrollTrigger.update
      });
      gsap.ticker.add((time) => {
        lenis.raf(time * 1000); // Convert time from seconds to milliseconds
      });
      gsap.ticker.lagSmoothing(0);
    }else{
      lenis && lenis.destroy();
    }
  },
  init(){ 
    this.resize(()=>{
      document.querySelectorAll(".has-round-bg").forEach(el=>{
        var cl = el.getBoundingClientRect();
        gsap.set(el, {
          '--wrap-height':cl.height+"px",
          '--wrap-width':cl.width+"px"
        });
      })
    }, true);
  },
  // END
  hero($scope){
    var eye = $scope.querySelector("svg.icon__eye");
    var path = $scope.querySelector(".line-svg .line-path");
    var duration =10;
    gsap.timeline({
      scrollTrigger:{
        trigger:$scope,
        start:`${eye.getBoundingClientRect().top} ${eye.getBoundingClientRect().top + 8}`,
        end:'bottom top',
        scrub:.4,
        // markers:true
      }
    })
    .to(eye, {
      amount:{
        from:'start'
      },
      motionPath:{
        path,
        align:path,
        alignOrigin:[1, 0], // <-- play with these numbers
        autoRotate:true,
        start: 1,
        end: 0
      },
      duration
    })
    .to(eye.querySelector(".c-wrap"), {
      rotation:-380,
      transformOrigin:"center",
      duration
    }, "<")
  },
  // HERO END
  Service($scope){ 
    var _this = this;
    var container = $scope.querySelector('.service-card-container');
    var inner = $scope.querySelector('.service-cards-inner');
    var wrap = $scope.querySelector('.service-cards-wrapper');
    var innerWidth = _this.client(inner).width;
    this.resize(()=>{
      gsap.set($scope, { 
        'height':(innerWidth + _this.client(wrap).height)+'px',
        // '--pin-top':Math.abs(_this.client($scope.querySelector('.service-cards-wrapper')).height - window.innerHeight)+"px"
      })
      let headerSpace = document.querySelector(".header__area").offsetHeight;
      let centerSpace = Math.abs(((window.innerHeight - _this.client(wrap).height) / 2));
      
      gsap.set(wrap, {
        '--pin-top':headerSpace > centerSpace ? headerSpace : centerSpace+"px"
      })
    }, true);
    gsap.to(inner, {
      '--x':'100%',
      '--container':(_this.client(container).width - 35)+'px',
      ease:"none",
      scrollTrigger:{
        trigger:$scope,
        start:`${_this.client(wrap).height/2} 50%`,
        end:'90% bottom',
        scrub:true,
        // markers:true
      }
    });
  },
  serviceCardsY($scope){
    var wrap = $scope.querySelector('.service-cards-y');
    var cardItems = $scope.querySelectorAll('.card__item');
    var secTitle = $scope.querySelector('.sec-title');
    var dime = this.getMaxDimensions(cardItems);
    gsap.set(wrap, {
      '--sticky-top':wrap.offsetTop+"px",
      '--max-h':dime.maxHeight+"px"
    })
    // gsap.to(secTitle, { 
    //   'margin-bottom':secMB+"px",
    //   scrollTrigger:{
    //     trigger:wrap,
    //     start:'top top',
    //     end:'bottom bottom',
    //     scrub:true
    //   }
    // }) 
     
    cardItems.forEach((item, index)=>{
      var card = item.querySelector('.service-card-full');
      gsap.set(card, {
        scale:.7,
        transformOrigin:'top center'
      })
      gsap.to(card, { 
        scale:1,
        ease:'none',
        scrollTrigger:{
          trigger:item,
          start:`top bottom`,
          end:`top +=${wrap.offsetTop}`,
          scrub:.1
        }
      });
      if(index > (cardItems.length-2)){ 
        gsap.to(secTitle, { 
          y:-wrap.offsetTop,
          ease:'none',
          scrollTrigger:{
            trigger:item,
            start:`top ${wrap.offsetTop}`,
            end:`top top`,
            scrub:true
          }
        });
      }
    })
    // ScrollTrigger.create({
    //   trigger:secTitle,
    //   start:`top ${secTitle.offsetTop}`,
    //   end:`+=${wrap.offsetTop + wrap.offsetHeight}`,
    //   pin:true,
    //   pinSpacing:false
    // })
    // console.log(cardItems.length);
    
    // cardItems.forEach((item, index)=>{
    //   var card = item.querySelector('.service-card-full');
    //   if(index < (cardItems.length-1)){
    //     gsap.to(card, {
    //       '--scale':1,
    //       scrollTrigger:{
    //         trigger:item,
    //         start:`top ${wrap.offsetTop}`,
    //         end:`+=${(wrap.offsetHeight - item.offsetTop) - cardItems[cardItems.length - 1].offsetHeight}`,
    //         scrub:true,
    //         markers:true,
    //         pin:true,
    //         pinSpacing:false
    //       }
    //     }) 
    //   }
    // })
  },
  // END
  splitLines(element, newEl = "div") {
    let _this = this;
    let lines = [];
    let innerLines = [];
    let newLine = new SplitType(element, { types: 'words,lines' });
    newLine.lines.forEach(line => {
       var wrap = _this.wrap(line, 'div', 'line-wrap');
       lines.push(wrap);
       innerLines.push(line);
    });
    return{
      lines,
      innerLines
    }
  },
  wrap(el, wrapper = "div", className = "word-wrap") {
      var wrapEl = document.createElement(wrapper);
      wrapEl.classList.add(className);
      el.parentNode.insertBefore(wrapEl, el);
      wrapEl.appendChild(el);
      return wrapEl;
  },
  // END
  position(childElement, parentElement){
    const childRect = childElement.getBoundingClientRect();
    const parentRect = parentElement.getBoundingClientRect();
  
    return {
      top: childRect.top - parentRect.top,
      left: childRect.left - parentRect.left,
      pw:parentRect.width,
      cw:childRect.width,
      ph:parentRect.height,
      ch:childRect.height
    };
  },
  client(el){
    return el.getBoundingClientRect();
  },
  resize(fun, callOnce = false, debounce = false){
    if(callOnce){
      fun();
      setTimeout(() =>fun(), 1000);
    };
    window.addEventListener('resize', fun);
  }, 
  // END
  autoScroll($scope){ 
    var teamWrap = $scope;
    var duration = teamWrap.getAttribute("data-speed") ? Number(teamWrap.getAttribute("data-speed")) : .6;  
    var InitialDirection = teamWrap.getAttribute("data-direction") && teamWrap.getAttribute("data-direction") === 'right' ? true : false;  
    
    for (let index = 0; index < 4; index++) {
        var child = teamWrap.querySelector(".scroll__item").innerHTML;
        var createWrap = document.createElement("div");
        teamWrap.appendChild(createWrap);
        createWrap.classList.add("scroll__item");
        createWrap.innerHTML = child;
    }

    var items = teamWrap.querySelectorAll('.scroll__item');
    var tl = this.horizontalLoop(items, {
        repeat:-1,
        speed:duration,
        reversed:InitialDirection
    });
    let direction = InitialDirection ? -1 : 1;
    if($scope.classList.contains('with-scroll')){
      ScrollTrigger.create({
        onUpdate(self){
          // console.log(self.getVelocity());
          
           if(InitialDirection){
            direction = self.direction === 1 ? -1 : 1;
           }else{
            direction = self.direction;
           }
            tl.timeScale(direction); 
        }
      })
    }
    teamWrap.addEventListener("mouseenter", function(){
      tl.paused();
      tl.timeScale(0);
    });
    teamWrap.addEventListener("mouseleave", function(){
      tl.play();
      tl.timeScale(direction); 
    });

  },
  // END
  horizontalLoop:function(items, config) {
    items = gsap.utils.toArray(items);
    config = config || {};
    let tl = gsap.timeline({repeat: config.repeat, paused: config.paused, defaults: {ease: "none"}, onReverseComplete: () => tl.totalTime(tl.rawTime() + tl.duration() * 100)}),
        length = items.length,
        startX = items[0].offsetLeft,
        times = [],
        widths = [],
        xPercents = [],
        curIndex = 0,
        pixelsPerSecond = (config.speed || 1) * 100,
        snap = config.snap === false ? v => v : gsap.utils.snap(config.snap || 1), // some browsers shift by a pixel to accommodate flex layouts, so for example if width is 20% the first element's width might be 242px, and the next 243px, alternating back and forth. So we snap to 5 percentage points to make things look more natural
        totalWidth, curX, distanceToStart, distanceToLoop, item, i;
    gsap.set(items, { // convert "x" to "xPercent" to make things responsive, and populate the widths/xPercents Arrays to make lookups faster.
        xPercent: (i, el) => {
            let w = widths[i] = parseFloat(gsap.getProperty(el, "width", "px"));
            xPercents[i] = snap(parseFloat(gsap.getProperty(el, "x", "px")) / w * 100 + gsap.getProperty(el, "xPercent"));
            return xPercents[i];
        }
    });
    gsap.set(items, {x: 0});
    totalWidth = items[length-1].offsetLeft + xPercents[length-1] / 100 * widths[length-1] - startX + items[length-1].offsetWidth * gsap.getProperty(items[length-1], "scaleX") + (parseFloat(config.paddingRight) || 0);
    for (i = 0; i < length; i++) {
        item = items[i];
        curX = xPercents[i] / 100 * widths[i];
        distanceToStart = item.offsetLeft + curX - startX;
        distanceToLoop = distanceToStart + widths[i] * gsap.getProperty(item, "scaleX");
        tl.to(item, {xPercent: snap((curX - distanceToLoop) / widths[i] * 100), duration: distanceToLoop / pixelsPerSecond}, 0)
          .fromTo(item, {xPercent: snap((curX - distanceToLoop + totalWidth) / widths[i] * 100)}, {xPercent: xPercents[i], duration: (curX - distanceToLoop + totalWidth - curX) / pixelsPerSecond, immediateRender: false}, distanceToLoop / pixelsPerSecond)
          .add("label" + i, distanceToStart / pixelsPerSecond);
        times[i] = distanceToStart / pixelsPerSecond;
    }
    function toIndex(index, vars) {
        vars = vars || {};
        (Math.abs(index - curIndex) > length / 2) && (index += index > curIndex ? -length : length); // always go in the shortest direction
        let newIndex = gsap.utils.wrap(0, length, index),
            time = times[newIndex];
        if (time > tl.time() !== index > curIndex) { // if we're wrapping the timeline's playhead, make the proper adjustments
            vars.modifiers = {time: gsap.utils.wrap(0, tl.duration())};
            time += tl.duration() * (index > curIndex ? 1 : -1);
        }
        curIndex = newIndex;
        vars.overwrite = true;
        return tl.tweenTo(time, vars);
    }
    tl.next = vars => toIndex(curIndex+1, vars);
    tl.previous = vars => toIndex(curIndex-1, vars);
    tl.current = () => curIndex;
    tl.toIndex = (index, vars) => toIndex(index, vars);
    tl.times = times;
    tl.progress(1, true).progress(0, true); // pre-render for performance
    if (config.reversed) {
      tl.vars.onReverseComplete();
      tl.reverse();
    }
    return tl;
  }, 
  // END
  addClass(element, className, c = true){
    var els = element;
    els.forEach(el=>c ? el.classList.add(className) : el.classList.remove(className));
  },
  element(string){ 
    return document.querySelector(string);
  },
  elements(string){
    return document.querySelectorAll(string);
  },
  // END
  splitingLines(el, by, toggle = this.toggleAnimations, trigger = {start:'top 90%',end:"+=100%",}){
    let _this = this;
    var reStart = toggle ? {toggleActions:"restart none none reverse"} : {};
    if(by === 'lines' || by === 'custom'){
      var {lines, innerLines} = this.splitLines(el);
      gsap.set(lines, {
        'overflow':'hidden'
      })
      gsap.set(innerLines, {
        y:"110%",
        rotation:7,
        transformOrigin:'0% 100%',
        // opacity:0
      })
      var toAni = {
        y:"0%",
        rotation:0,
        ease:_this.defaults.ease,
        duration:_this.defaults.duration,
        stagger:0.1,
      }
      if(by === 'custom'){
        return {lineTl:gsap.timeline({paused:true}).to(innerLines, {...toAni}), lines, innerLines}
      }
      gsap.to(innerLines, { 
        ...toAni,
        scrollTrigger:{
          trigger:el,
          ...trigger,
          // markers:true,
          ...reStart
        }
      });
    }
    if(by === 'words'){
      let text = new SplitType(el,{types:'words'});
      gsap.set(text.words, {
        y:"110%",
        opacity:0
      })
      gsap.to(text.words, { 
        y:"0%",
        opacity:1,
        ease:'expo.out',
        duration:.8,
        stagger:0.01,
        scrollTrigger:{
          trigger:el,
          ...trigger,
          // markers:true,
          ...reStart
        }
      });
    }
    el.querySelectorAll(".absolute").forEach(line=>{
      gsap.set(line, {'position':""});
    });
  },
  // END
  clippingMasks(el, ani = ""){
    let _this = this;
    var reStart = this.toggleAnimations ? {toggleActions:"restart none none reverse"} : {};
    var centerClip = el.classList.contains("center__clip_In");
    gsap.set(el, {
      '--c':centerClip ? "50%" :"0%"
    })
    let toAni = {
      '--c':centerClip ? "0%" : "100%",
      ease:"power2.inOut",
      duration:1,
    }
    if(ani === 'custom'){
      return gsap.timeline({paused:true}).to(el, {...toAni})
    }
    gsap.to(el, { 
      ...toAni,
      scrollTrigger:{
        trigger:el,
        start:'top 95%',
        end:"+=100%",
        // markers:true,
        ...reStart
      }
    });
  },
  // END
  footer(){
    var footerWrap = document.querySelector("#site-footer");
    if(footerWrap){
      var title = footerWrap.querySelector(".title");
      var aniNames = footerWrap.querySelectorAll("[data-ani-name]");
      var _this = this;
      this.resize(()=>{
        gsap.set(footerWrap, {
          '--height':_this.client(footerWrap).height+"px",
          '--bg-rotation':"-80deg"
        })
      }, true);
      if(footerWrap){
        let {lineTl, lines} = _this.splitingLines(title, "custom");
        let clip = _this.clippingMasks(title, "custom");
        footerWrap.querySelectorAll('[data-inline-motion]').forEach(el=>{
          _this.motion(el, true, {
            trigger:footerWrap, 
            start:'bottom 110%'
          })
        });
        ScrollTrigger.create({
          trigger:footerWrap, 
          start:'bottom 110%',
          onEnter(){
            gsap.set(lines, {'overflow':'hidden'});
            lineTl.play();
            setTimeout(() => {
              gsap.set(lines, {'overflow':''});
              clip.play();
            }, (_this.defaults.duration + 1000) / 2);
          },
          onLeaveBack(){
            gsap.set(lines, {'overflow':'hidden'});
            lineTl.reverse();
            clip.reverse();
          }
        })
        gsap.to(footerWrap, {
          '--bg-scale':"100%",
          '--bg-rotation':"0deg",
          scrollTrigger:{
            trigger:footerWrap, 
            start:'top 95%',
            end:"bottom bottom",
            scrub:true,
            // markers:true,
            toggleActions: "restart none none reverse",
            onEnter(){
  
            },
            onEnterBack(){
  
            }
          }
        })
      }
    }
  },
  button($scope){
    var hasArrow = $scope.classList.contains('btn-hover-arrow');
    var _this = this;
    if(hasArrow){
      this.resize(function(){
        gsap.set($scope, {'--l':_this.position($scope.querySelector('.btn-text'), $scope).left+"px"});
      })
    }
  },
  // END
  cssAni($scope, toggle = this.toggleAnimations, srigger = {trigger:$scope, start:'top 100%',end:"+=100%"}){
    var aniNameCheckk = $scope.getAttribute('data-ani') ? $scope.getAttribute('data-ani') : $scope.getAttribute('data-ani-name');
    var aniName = $scope.getAttribute('data-ani-name2') ? $scope.getAttribute('data-ani-name2') : aniNameCheckk;
    var CSSDuration = $scope.getAttribute('data-ani-duration') ? {'--duration':$scope.getAttribute('data-ani-duration')} : "";
    var CSSOrgin = $scope.getAttribute('data-ani-orgin') ? {'--transformOrgin':$scope.getAttribute('data-ani-orgin')} : "";
    let hasOverflow = null;
    function removeOverflow(status = "enter"){
      hasOverflow = $scope.parentElement;
      if(hasOverflow.classList.contains('overflow-ani')){
        if(status === 'enter'){
          setTimeout(() => {
            hasOverflow.classList.remove('overflow-ani');
          }, 1000);
        }
      }
    }
    let start = ()=>{
      gsap.set($scope, {
        'visibility':'visible',
        'animation-name':aniName,
        ...CSSDuration,
        ...CSSOrgin,
      })
    }
    let reSet = ()=>{
      gsap.set($scope, {
        'visibility':'',
        'animation-name':""
      })
    }
    if(!$scope.parentElement.classList.contains('data-ani-stagger')){
      ScrollTrigger.create({
        ...srigger,
        onEnter(){
          start();
          removeOverflow();
        },
        onLeaveBack(){ 
          if(toggle) reSet();
        }
      })
    }
    if($scope.classList.contains('data-ani-stagger')){
      ScrollTrigger.create({
        ...srigger,
        onEnter(){
          var delay = 0;
          $scope.querySelectorAll('[data-ani]').forEach((item)=>{
            var aniName = item.getAttribute('data-ani');
            setTimeout(() => {
              start();
            }, delay);
            delay+=120
          })
        }
      })
    }
    return{
      start,
      reSet
    }
  },
  motion($scope, toggle = this.toggleAnimations, trigger = {trigger:$scope, start:"top 90%", end:"+=100%"}){
    var name = $scope.getAttribute('data-inline-motion');
    var reStart = toggle ? {toggleActions:"restart none none reverse"} : {};
    let STrigger = {scrollTrigger:{...trigger,...reStart}};
    if(name === 'rotateFadeIn'){
      gsap.set($scope, {
        autoAlpha:0,
        y:120,
        rotate:4
      })
      gsap.to($scope, {
        autoAlpha:1,
        rotate:0,
        y:0,
        ...STrigger
      })
    }
  },
  // END
  parentAni($scope){
    var data = $scope.getAttribute("data-parent-ani") && $scope.getAttribute("data-parent-ani") != "" ? eval('('+$scope.getAttribute("data-parent-ani")+')') : false;  
    var fixedSection = data.fixed && data.fixed != "" ? document.querySelector(data.fixed) : false;
    var toLeft = data.toLeft && data.toLeft != "" ? document.querySelector(data.toLeft) : false;
    var toFadeOutUp = data.toFadeOutUp && data.toFadeOutUp != "" ? document.querySelector(data.toFadeOutUp) : false;
    var toUp = data.toUp && data.toUp != "" ? document.querySelector(data.toUp) : false;
    var sObj = {
      trigger:$scope,
      start:window.innerWidth > 991 ? '-150 bottom' : 'top bottom', 
    }
    if(fixedSection){
      var parentScroll = ScrollTrigger.create({
        ...sObj,
        end:"top top",
        pin:fixedSection,
        pinSpacing:false,
        pinType: "fixed",
        // markers:true
      })
    }
    var tl = gsap.timeline({
        defaults:{
          ease:'none'
        },
        scrollTrigger:{
          ...sObj,
          end:"top 10%", 
          scrub:true,
        }
    })
    toLeft && tl.to(toLeft, {
      x:"-100%"
    })
    toFadeOutUp && tl.to(toFadeOutUp, { 
      y:"-150%",
      opacity:0
    }, "<")
    toUp && tl.to(toUp, { 
      y:-400
    }, "<")
  }, 
  process($scope){
    var _this = this;
    var inner = $scope.querySelector(".process-inner-section");
    var items = $scope.querySelectorAll('.process-item');
    var bottomBtn = $scope.querySelector(".bottom-btn");
    //  var getItemsDimension = _this.getMaxDimensions(items);
    this.resize(()=>gsap.set($scope.querySelector('.process-wrapper'), {"--height":_this.getMaxDimensions(items).maxHeight+"px"}), true);

    gsap.timeline({
      scrollTrigger:{
        trigger:inner, 
        start:window.innerWidth > 991 ? 'top 95%' : "top 110%",
        end:window.innerWidth > 991 ? "bottom bottom" : "top top",
        scrub:true,
        toggleActions: "restart none none reverse",
        // markers:true,
      }
    })
    .to(inner, {
      '--bg-scale':"100%",
      // '--radius':"0%",
      ease:"none",
      duration:1
    })
    ScrollTrigger.create({   
      trigger:inner,
      start:"top top",
      onEnter(){
        gsap.to(inner, {
          '--radius':"0%",
          ease:"expo.out",
          duration:1
        })
      },
      onLeaveBack(){
        gsap.to(inner, {
          '--radius':"50%",
          ease:"expo.out"
        })
      }
    })
    // PC Device
    let itemsArray = [];
    items.forEach((el, index)=>{  
      var contentLeft = el.querySelector(".left-content");
      var contentRight = el.querySelector(".right-content");
      var currentSlide = el.querySelector(".card__slide") ? _this.initSlide(el.querySelector(".card__slide")) : false; if(currentSlide) currentSlide.autoplay.stop();
      var currentResponsiveSS = el.querySelector(".responsive-ss") ? _this.responsiveSS(el.querySelector(".responsive-ss")) : false;
      itemsArray.push({
        slide:currentSlide,
        responsiveSS:currentResponsiveSS
      });
      if(1 > index){
        gsap.set(contentLeft, {
          x:-300,
          opacity:0
        })
        gsap.set(contentRight, {
          x:300,
          autoAlpha:0
        })
        var openContentTl = gsap.timeline({paused:true}).to([contentLeft,contentRight], {
          x:0,
          autoAlpha:1,
          ease:"expo.out",
          duration:1
        })
        ScrollTrigger.create({
          trigger:el,
          start:"top 85%",
          end:"+=100%",
          // markers:true,
          onEnter(){
            openContentTl.play();
            if(currentSlide){
              setTimeout(() => { 
                currentSlide.autoplay.resume()
                currentSlide.autoplay.start();
              }, 200);
            };
            var processWrap = $scope.querySelector('.process-wrapper');
            gsap.set(processWrap, {marginTop:window.getComputedStyle(processWrap).marginTop})
          },  
          onLeaveBack(){
            openContentTl.reverse();
            if(currentSlide) currentSlide.autoplay.stop();
            var processWrap = $scope.querySelector('.process-wrapper');
            gsap.set(processWrap, {marginTop:""})
          }
        })
        return;
      }
      
      var prevEl = items[index - 1].querySelector('.process-inner'),
          prevTitle = prevEl.querySelector(".title > span"),
          prevSubTitle = prevEl.querySelector(".sub-title"),
          prevDes = prevEl.querySelector(".des > div"),
          prevMedia = prevEl.querySelector(".media__con"),
          currentEl = el.querySelector(".process-inner"), 
          currentTitle = currentEl.querySelector(".title > span"),
          currentSubTitle = currentEl.querySelector(".sub-title"),
          currentDes = currentEl.querySelector(".des > div"),
          currentMedia = currentEl.querySelector(".media__con"),
          currentLeft = currentEl.querySelector(".left-content"),
          currentRight = currentEl.querySelector(".right-content");  
          
          _this.resize(()=>{  
            var position = _this.position(el, $scope.querySelector('.process-wrapper'));
            var maxH = _this.getMaxDimensions(items).maxHeight;
            gsap.set(currentEl, {y:-position.top}); 
            if((index+2) > items.length){
              // gsap.set($scope,{'--h':(window.innerHeight+position.top)+"px"});
              gsap.set($scope,{'--h':(window.innerHeight+position.top + (el.classList.contains('zoom-screen-item') ? position.ch * 4 : position.ch))+"px"});
            }
          }, true);

      gsap.set(currentSubTitle, {autoAlpha:0})
      let AnimationTl = (enter = true)=>{
          var currentAutoAlpha = enter ? 1 : 0;
          var currentRotation = enter ? 0 : 8;
          var prevAutoAlpha = enter ? 0 : 1;

          // Previous Animation
          gsap.to([prevTitle, prevDes], {
            y:enter ? "-100%" : "0%",
            rotation:0
          });
          gsap.to(prevSubTitle, {autoAlpha:prevAutoAlpha});
          gsap.to(prevMedia, {
            scale:enter ? 0.5 : 1, 
            autoAlpha:prevAutoAlpha
          });
          var prevSesponsiveSS = itemsArray[index - 1].responsiveSS;
          if(prevSesponsiveSS && !enter){prevSesponsiveSS.restart()};

           // Current Animation
          gsap.to([currentTitle, currentDes], {
            y:enter ? "0%" : "100%",
            rotation:currentRotation
          });
          gsap.to(currentSubTitle, {autoAlpha:currentAutoAlpha, duration:.3});
          gsap.to(currentMedia, {
            x:enter ? "0%" : "100%", 
            autoAlpha:currentAutoAlpha,
            scale:enter ? 1 : 0.5
          });

          var PrevSlide = itemsArray[index - 1].slide;
          if(PrevSlide){ enter ? PrevSlide.autoplay.stop() : PrevSlide.autoplay.start()}; 
          if(currentSlide){ enter ? PrevSlide.autoplay.start() : PrevSlide.autoplay.stop()}; 
          if(currentResponsiveSS && enter){currentResponsiveSS.restart()};

      }     
      let setAnimation = ()=>{
        gsap.set([currentTitle, currentDes], {y:"100%", rotation:8, transformOrigin:"0% 0%"})
        gsap.set(currentMedia, {x:"100%", scale:0.5, autoAlpha:0}) 
        // gsap.set([currentEl, currentSubTitle], {autoAlpha:0})
        gsap.set(currentMedia, {autoAlpha:0})
      }
      ScrollTrigger.create({ 
        trigger:el,
        start:window.innerWidth > 991 ? "top 90%" : "top 40%",
        end:"bottom 50%",
        // markers:true, 
        onEnter(){
          AnimationTl(true);
          _this.addClass(items, 'active', false);
          _this.addClass([el], 'active',true);

        },
        onLeaveBack(){ 
          AnimationTl(false); 
          _this.addClass(items, 'active', false);
          _this.addClass([items[index - 1]], 'active',true);
        }
      });  
      setAnimation();

      if(el.classList.contains('zoom-screen-item')){ 
        let screenTl = gsap.timeline();
        let clonedEl;
        let elColonReplace = (status = 'enter')=>{
          let enter = status === 'enter';
          let clip = enter ? 'inset(0 var(--clip) 0 0)' : 'inset(0 0 0 var(--clip))';
          let bPos = enter ? {"right":"var(--clip)"} : {"left":"var(--clip)"};
          let fromContent = el.querySelector('.from__content');
          let toContent = window.innerWidth > 991 ? $scope.querySelector('.to__content') : el.querySelector('.to__content_mobile');
          let parentSection = window.innerWidth > 991 ? $scope.querySelector('.process-inner-section') : el.querySelector(".right-content");
          let zoomScreen = window.innerWidth > 991 ? $scope.querySelector('.zoom-screen') : el.querySelector('.zoom-screen-mobile');
          let fromClon = status === 'enter' ? fromContent : toContent;
          let toReplace = status === 'enter' ? toContent : fromContent;
          let replacerdEl = document.createElement('div');
          let replacerdBorderEl = document.createElement('div');
          let relativeEl = window.innerWidth > 990 ? false : el.querySelector(".right-content");
          let rightContent = el.querySelector('.right-content');
          clonedEl = _this.cloneContent(fromClon, parentSection, {
            'z-index':100,
          }, relativeEl);
          gsap.set(replacerdBorderEl, {'position':'absolute', 'height':'100%','width':'8px', 'background-color':"#FFCC54", "top":0, ...bPos});  
          // Replace Start
          replacerdEl.innerHTML = toReplace.innerHTML;
          clonedEl.appendChild(replacerdEl);
          replacerdEl.appendChild(replacerdBorderEl);
          gsap.set(replacerdEl, {
            'position':'absolute', 'top':0,'left':'0', 'z-index':555, 'width':'100%','height':'100%','--border':'0px', '--clip':'100%', 'clip-path':clip, 'z-index':5
          })
          gsap.to(replacerdEl, {
            '--clip':"0%",
            onComplete(){
              gsap.to(replacerdBorderEl, {
                'width':'0px',
                duration:.2
              }) 
            }
          })  
          // Replace End
          _this.pestContent(clonedEl, toReplace, {}, ()=>{
            gsap.set(toReplace, {autoAlpha:1});  
            if(status === 'enter'){
              _this.addClass([zoomScreen], 'v-hidden', false); 
            }else{
              _this.addClass([zoomScreen], 'v-hidden');  
            }
            clonedEl.remove();
          }, relativeEl);
          var styleForLastLeft = window.innerWidth > 991 ? {x:enter ? "50%" : "0%"} : {y:enter ? "50%" : "0%",};
          gsap.to(currentLeft, {
            ...styleForLastLeft,
            opacity:enter ? 0 : 1,
          })
          gsap.to(window.innerWidth > 991 ? currentRight : currentRight.querySelectorAll('.fadeOutforAfter'), {
            x:enter ? "-50%" : "0%",
            opacity:enter ? 0 : 1,
            onComplete(){
              gsap.set(currentRight, {x:0});
            }
          });
          if(window.innerWidth < 991){
            gsap.to(rightContent, {
              y:status === 'enter' ? (window.innerHeight / 2) - (_this.client(rightContent).top + (rightContent.offsetHeight / 2)) : 0,
              '--bg-size':status === 'enter' ? "130%" : "100%",
              duration:1,
              ease:"expo.out"
            })
          } 
        }
        ScrollTrigger.create({ 
          trigger:el,
          start:window.innerWidth > 991 ? "250% 50%" : "210% 70%",
          // markers:true, 
          onEnter(){
            if(window.innerWidth > 991){
              elColonReplace('enter');
            }else{
              setTimeout(() => {
                elColonReplace('enter');
              }, 100);
            }
          },
          onLeaveBack(){ 
            if(window.innerWidth > 991){
              elColonReplace('back');
            }else{
              setTimeout(() => {
                elColonReplace('back');
              }, 100);
            }
          }
        });  
      }
    });
    // Mobile Device

  }, 
  cloneContent(content, appendEl, ex = {}, relative = false){
      let _this = this;
      let relativeV = false;
      if(relative){
        relativeV = this.getRelativeOffset(content, relative);
      }
      var bound = content.getBoundingClientRect();
      var clonedElement = appendEl.querySelectorAll('.content__cloned');
      let createClon = clonedElement.length && clonedElement[0];
      if(!clonedElement.length){  
          createClon = document.createElement('div');
          createClon.classList.add('content__cloned');
          createClon.innerHTML = content.outerHTML;
          appendEl.appendChild(createClon);
      }
      gsap.set(content, {    
          opacity:0   
      })
      gsap.set(createClon, {  
          'top':relativeV ? relativeV.top : bound.top,
          'left':relativeV ? relativeV.left : bound.left,
          'width':bound.width,
          'height':bound.height,
          'position':'absolute',
          ...ex
      });
      return createClon;
  },
  pestContent(fromEl, toEl, tl = {}, fun = ()=>{}, relative = false){  
      let relativeV = false;
      if(relative){
        relativeV = this.getRelativeOffset(toEl, relative);
      }
      var bound = toEl.getBoundingClientRect();
      gsap.to(fromEl, {
          'top':relativeV ? relativeV.top : bound.top,
          'left':relativeV ? relativeV.left : bound.left,
          'width':bound.width,
          'height':bound.height,
          ...tl,
          onComplete(){
              fun(fromEl, toEl); 
          }
      }) 
  },
  getRelativeOffset(child, parent) {
      if (!child || !parent) return null;
      const childRect = child.getBoundingClientRect();
      const parentRect = parent.getBoundingClientRect();
      return {
          top: childRect.top - parentRect.top,
          left: childRect.left - parentRect.left
      };
  },
  introSection($scope){
      var _this = this;
      var headingWrap = $scope.querySelector(".heading-wrap-inner");
      var headingContent = headingWrap.querySelector(".heading-content");
      var headingBG = $scope.querySelector(".heading__bg");
      var inner = $scope.querySelector(".intro-content-inner");
      var ContentArea = $scope.querySelector(".intro-content-area");
      var contentInner = $scope.querySelector(".content-inner");
      var eye = $scope.querySelector(".icon__eye");
      var icons = headingWrap.querySelectorAll(".icons");
      var piningProgress = $scope.querySelector('.pining__progress');
      var contentItemsWrap = $scope.querySelector('.intro-content-items'); 
      var contentItems = $scope.querySelectorAll('.content-item');
      var eyeWidth =  this.client(eye).width;
      gsap.set(headingWrap, {
        '--x-plus':-eyeWidth/2+'px'
      });
      var startIntroProgress = 20;
      // FIXED ELEMENT
      if(headingWrap){
        ScrollTrigger.create({
          trigger:piningProgress,
          start:'top bottom',
          end:window.innerWidth > 991 ? `350% top` : 'top top',
          pin:headingWrap,
          pinSpacing:false,
          // markers:{
          //   startColor:'blue',
          //   endColor:'blue'
          // }
        });
        var IBox = gsap.to(headingWrap,{
          '--x':"0%",
          '--rotation':"280deg", 
          '--x-plus':0,
          '--ix':0,
          '--iy':0,
          '--ix2':0,
          ease:"none", 
          // markers:true,
          scrollTrigger:{
            trigger:piningProgress,
            start:'top bottom',
            end:window.innerWidth > 991 ? "top top" : "top 20%",
            scrub:.1,
            onLeave(){
              _this.addClass([$scope],'bg-blue')
            },
            onEnterBack(){
              _this.addClass([$scope],'bg-blue', false); 
            },
            // onLeave
          }
        })
        var IBoxLater = gsap.to(headingWrap,{  
          '--icon-x':"100%",
          ease:"none", 
          scrollTrigger:{
            trigger:piningProgress,
            start:window.innerWidth > 991 ? 'bottom 20%' : 'top 20%',
            end:window.innerWidth > 991 ? "bottom top" : 'top top',
            scrub:.4, 
            // markers:true,
            onLeave(){
              // console.log("On Leave");
            }
          }
        })   
        ScrollTrigger.create({
          trigger:headingBG,
          start:"right center",
          end:"left left",
          containerAnimation:IBox,
          onEnter(){
            _this.addClass([headingContent], 'active');
          }, 
          onLeaveBack(){
            _this.addClass([headingContent], 'active', false);
          }
        })
      }
      // CONTENT ANIMATIONS
      var createTrackWrapEl = document.createElement('div');
      createTrackWrapEl.classList.add('intro-ani-track-wrap');
      inner.appendChild(createTrackWrapEl);
      gsap.set(createTrackWrapEl, {
        'position':'absolute',
        'left':0,
        'top':0,
        'width':"100%",
        'pointer-events':'none'
      });
      contentItems.forEach((item, index)=>{
        
        var createItemElWrap = document.createElement('div');
        var trackElements = [];
        var slideUp = item.querySelectorAll('.scroll-slide-up');
        createTrackWrapEl.appendChild(createItemElWrap);
        slideUp.forEach(()=>{
          var trackEl = document.createElement('div'); trackEl.classList.add('intro-ani-track-item'); createItemElWrap.appendChild(trackEl);
          trackElements.push(trackEl);
        })
        var title = item.querySelector('.inner-title');
        var description = item.querySelector('.inner-des');
        gsap.set([title,description], {
          y:"100%",
          rotation:5,
          transformOrigin:'0 100%'
        });
        gsap.set(slideUp,{'y':'100vh'})
        _this.resize(()=>{
          var imgFlow = item.querySelector('.img__flow');
          var client = _this.client(imgFlow);
          imgFlow.querySelectorAll('.fadeInCenter').forEach(el=>{
            var position = _this.position(el, imgFlow);
            var x = (client.width / 2) - position.left;
            var y = (client.height / 2) - position.top;
            gsap.set(el, {x, y, opacity:0});
          })
        }, true);

        ScrollTrigger.create({ 
          trigger:createItemElWrap,
          start:"top top",
          // markers:true, 
          onEnter(){
            _this.addClass(contentItems, 'active', false);
            _this.addClass([item], 'active',true);
          },
          onLeaveBack(){ 
            _this.addClass(contentItems, 'active', false);
            contentItems[index - 1] && _this.addClass([contentItems[index - 1]], 'active',true);
          }
        }); 

        slideUp.forEach((slideUpEl, index)=>{     
          var fadeInCenter = item.querySelectorAll('.fadeInCenter');
          var cc = item.querySelectorAll('.custom-has-cl-ani');
          var fadeUpEl = item.getAttribute('data-slideOut-parent') ? document.querySelector(item.getAttribute('data-slideOut-parent')) : false;
          var common = {
            ease:'expo.InOut',
            scrollTrigger:{
              trigger:trackElements[index],
              start:'top top',
              end:'bottom top',
              scrub:1,
              // markers:true
            }
          }
          gsap.to(slideUpEl, {
            y:0,
            ...common
          }) 
          if(index < 1){   
            gsap.to([title,description], {
              y:0,
              rotation:0,
              ...common
            })
            if(fadeInCenter){
              gsap.to(fadeInCenter, {
                y:0,
                x:0,
                opacity:1,
                ...common
              })
            }
            if(fadeUpEl){
              gsap.to(fadeUpEl, {
                y:"-100vh",
                ...common
              })
            }
            if(cc){
              gsap.to(cc, {
                '--c':"100%",
                ...common
              })
            }
          }
        })
        _this.resize(()=>{   
          var position = _this.position(item, contentItemsWrap);
          var maxH = _this.getMaxDimensions(contentItems).maxHeight;
          gsap.set(item, {'--space-y':-position.top+"px"});
          // console.log(maxH);
          gsap.set(contentItemsWrap, {'--ch':maxH+"px"});
        }, true);
      });
      _this.resize(()=>{
        gsap.set(ContentArea, {'--ex-h':_this.client(ContentArea.querySelector('.intro-ani-track-wrap')).height+"px"});
      }, true);
  },
  teamWrap($scope){
    var cols = $scope.querySelectorAll('.team-col');
    var cards = $scope.querySelectorAll('.team-card');
    gsap.set(cols, {
      y:window.innerHeight/2
    })
    gsap.to(cols, {
      y:0,
      stagger:.1,
      duration:1,
      ease:'power2.out',
      scrollTrigger:{
        trigger:$scope,
        start:'top bottom',
        end:'top 40%',
        scrub:.5,
        // markers:true
      }
    })
  },
  responsiveSS($scope){
    var desktop = $scope.querySelector(".desktop");
    var mobile = $scope.querySelector(".mobile");
    var line = $scope.querySelector(".line");
    gsap.set(line, {'--lt':"0%", '--lb':"10%"});
    gsap.set(desktop, {
      scale:.4,
      y:60,
      opacity:0
    })
    gsap.set(mobile, {
      x:-20,
      opacity:0
    })
    return gsap.timeline({paused:true, default:{ease:"power1.out", duration:.5}}).to(desktop,{scale:1,y:0,opacity:1}).to(line,{'--lt':"100%", duration:.2}).to(line,{'--lb':"100%", duration:.2}).to(mobile,{x:0,opacity:1})
  },
  getMaxDimensions(elements) { 
    var elArray = [];
    const dimensions = Array.from(elements).map(el =>{
      elArray.push({
        height: el.offsetHeight,
        width: el.offsetWidth
      })
      return {
        height: el.offsetHeight,
        width: el.offsetWidth
      }
    });
    let maxHeight = Math.max(...dimensions.map(dim => dim.height));
    let maxWidth = Math.max(...dimensions.map(dim => dim.width));
    return { maxHeight, maxWidth, elArray };
  },
  initSlide:function($scope){
    var _this = this;
    var Settings = $scope.getAttribute("data-settings") && $scope.getAttribute("data-settings") != "" ? eval('('+$scope.getAttribute("data-settings")+')') : false;  
    var slide = $scope.querySelector(".swiper");
    var lg = Settings.lg !== undefined && Settings.lg != "" ? Settings.lg : false;
    var md = Settings.md !== undefined && Settings.md != "" ? Settings.md : false;
    var sm = Settings.sm !== undefined && Settings.sm != "" ? Settings.sm : false;   
    var loop = Settings.loop !== undefined && Settings.loop != "" ? Settings.loop : false;   
    var autoPlay = Settings.autoPlay !== undefined && Settings.autoPlay != "" ? Settings.autoPlay : false;
    var delay = Settings.delay !== undefined && Settings.delay != "" ? Settings.delay : 500;
    var zoom = $scope.classList.contains("slide-zoom");
    let stroge = {};
    let numSlidePagination = $scope.querySelector(".slide-num-pagination");
    var _pagination = $scope.querySelector(".slide-pagination") ? {
      pagination:{
          el:$scope.querySelector(".slide-pagination"),
          clickable:true
      }
    }:{};
    var _NumPagination = numSlidePagination ? {
      pagination:{
          el:$scope.querySelector(".slide-num"),
          clickable:true,
          renderBullet: function (index, className) {
            $scope.querySelector('.slide-total').textContent = stroge.totalSLide > 10 ? stroge.totalSLide : '0'+stroge.totalSLide;
            var num = (index + 1) > 10 ? (index + 1) : "0"+(index + 1);
            return '<span class="' + className + '" data-index="'+index+'">'+num+'</span>';
          }
      }
    }:{};
    let numPFun = ()=>{
      if(numSlidePagination){
        var active = numSlidePagination.querySelector('.swiper-pagination-bullet-active');
        var bulets = numSlidePagination.querySelectorAll('.swiper-pagination-bullet');
        var height = _this.getMaxDimensions(bulets);
        gsap.set(numSlidePagination.querySelectorAll('.swiper-pagination-bullets'), {'--mh':height.maxHeight+"px", '--active-index':active.getAttribute('data-index')});
      }
    }
    this.resize(()=>numPFun());
    var _navigation = $scope.querySelector(".slide-nav") ? {
      navigation: {
        nextEl: $scope.querySelector(".slide-next"),
        prevEl: $scope.querySelector(".slide-prev"),
      }
    }:{}; 
    var slideEffect=zoom?{
      effect: 'creative',
      creativeEffect: { 
          limitProgress: 10,
          prev: {
            shadow: false,
            opacity: 0,
            scale: 0.5,
            /* translate(<length-percentage> , <length-percentage>?)*/
            translate: ["-70%", 0, 0],
            rotate:[0,0,-4]
          },
          next: {
            shadow: false,
            opacity:0,
            scale: 0.5,
            translate: ["50%", 0, 0],   
            rotate:[0,0,4]
          },
      }
    }:{};
    let SwiperZoomSlide = new Swiper(slide, {  
      slidesPerView:lg ? lg : 1, 
      loop, 
      autoplay:autoPlay ? {       
        delay
      } : false, 
      speed:800,  
      ...slideEffect,
      ..._pagination, 
      ..._NumPagination, 
      ..._navigation,
      on:{
          touchEnd:function(swiper, event){
          },
          init:function(swiper){ 
            numPFun();
          },
          beforeInit: function () {
            stroge.totalSLide = this.wrapperEl.querySelectorAll(".swiper-slide").length;
          },
          slideChange(){
            numPFun();
          }
          // slideChange(){
            
          // },
      }, 
      breakpoints:{   
          0:{
              slidesPerView:sm ? sm : 1,
              spaceBetween:13
          },
          768:{
              slidesPerView:md ? md : 1,
              spaceBetween:15
          },
          991:{
              slidesPerView:lg ? lg : 1,
              spaceBetween:25
          }
      }
    }); 
    return SwiperZoomSlide;
  },
  drawSVG(path, duration = 2000){
    const pathLength = path.getTotalLength();
    path.style.strokeDasharray = pathLength;
    path.style.strokeDashoffset = pathLength;
    let start = null;
    function animate(time) {
      if (!start) start = time;
      const progress = time - start;
      const offset = Math.max(pathLength - (progress / duration) * pathLength, 0);
      path.style.strokeDashoffset = offset;
      if (progress < duration) {
        requestAnimationFrame(animate);
      }
    }
    return{
      play:()=>requestAnimationFrame(animate)
    }
  },
  click(selector, fun){
    selector.addEventListener('click', fun);
  },
  innerWidhAni($scope){
    var _this = this;
    var innerWidth = $scope.querySelector("span");
    gsap.set(innerWidth, {
      'min-width':_this.client(innerWidth).width+"px"
    })
    gsap.set($scope, {
      width:0,
      'overflow':'hidden'
    })
    return{start(tl = {ease:'back.out(4)'}){
      gsap.to($scope, {
        width:'auto',
        ...tl,
        onComplete(){
          gsap.set($scope, {
            'overflow':'',
            width:'',
          })
          gsap.set(innerWidth, {
            'min-width':""
          })
        }
      })
    }}
  },
  pinedContent($scope){
    var _this = this;
    var parentEl = $scope.closest('.has-custom-pin');
    this.resize(()=>{
      var exHeight = parentEl.classList.contains("ex-screen") ? window.innerHeight : 0;
      gsap.set(parentEl, {'--h':(_this.client($scope).height+exHeight)+"px"}); 
    }, true);
  },
  aboutHero($scope){
    var _this = this;
    var topTextLeftAni = $scope.querySelector('.title [data-ani-name="slideInLeft"]');

    setTimeout(() => {
      _this.cssAni(topTextLeftAni);
    }, 520);
    setTimeout(() => {
      $scope.querySelectorAll('.bg-patterns [data-ani-name]').forEach(el=>_this.cssAni(el));
    }, 800);
    $scope.querySelectorAll('.title .innerWidthAni').forEach(el=>{
      let ina = _this.innerWidhAni(el);
      setTimeout(() => {
        ina.start();
        $scope.querySelectorAll('.bg-patterns [data-ani-name]').forEach(el=>_this.cssAni(el));
      }, 800);
    });
  },
  drawLogo($scope){
    var logo = $scope.querySelector('.main__logo');
    var lines = $scope.querySelector('.lines');
    var reStart = this.toggleAnimations ? {toggleActions:"restart none none reverse"} : {};
    gsap.set(logo, {
      opacity:0
    })
    gsap.to(logo, {
      opacity:1,
      duration:1,
      scrollTrigger:{
        trigger:$scope,
        start:'top 50%',
        ...reStart,
      }
    })
  },
  animatedNumber($scope){
    let _this = this;
    var reStart = this.toggleAnimations ? {toggleActions:"restart none none reverse"} : {};
    var splite = $scope.textContent.split('');
    let el;
    // $scope.innerHTML = "00";
    let height = $scope.offsetHeight;
    $scope.innerHTML = "";
    splite.forEach(item=>{
      var createElement = document.createElement('div');
      if(item != ','){
        gsap.set(createElement, {
          height:height
        });
        for(let index = 0; index < 10; index++){
          var createInnerElement = document.createElement('div');
          createInnerElement.textContent = index;
          gsap.set(createInnerElement, {
            "height":"100%"
          })
          createElement.appendChild(createInnerElement);
        }
      }
      var createLastElement = document.createElement(item === ',' ? 'span' : 'div');
      createLastElement.textContent = item;
      createElement.classList.add('parent__number__el');
      createLastElement.classList.add('last__number__el');
      item === ',' && createElement.classList.add('comma');
      gsap.set(createLastElement, {
        'display':'inline-block',
        "height":"100%",
      });
      createElement.appendChild(createLastElement);
      $scope.appendChild(createElement);
    });
    var coultEl = $scope.querySelectorAll("div > div");
    $scope.querySelectorAll('.last__number__el').forEach(el=>{
      gsap.to(el.closest('.parent__number__el'), {
        'width':_this.client(el).width+"px",
        ease:"power2.out",
        duration:1.5,
        scrollTrigger:{
          trigger:$scope,
          start:'top 95%',
          ...reStart,
        }
      })
    });
    gsap.set($scope,{
      'overflow':'hidden'
    })
    gsap.to(coultEl, {
      yPercent:-1000,
      stagger:0.01,
      ease:"power2.out",
      duration:1.5,
      scrollTrigger:{
        trigger:$scope,
        start:'top 95%',
        ...reStart,
      }
    });
  },
  textWaveIn($scope){
    var duration = this.defaults.duration;
    var {lines} = new SplitType($scope, { types: 'lines' });
    var toggle = this.toggleAnimations;
    var reStart = toggle ? {toggleActions:"restart none none reverse"} : {};
    $scope.querySelectorAll(".absolute").forEach(line=>{
      gsap.set(line, {'position':""});
    });
    gsap.set(lines,{
      y:200,
      z:-41,
      opacity:0,
      rotationX:-80,
      rotationY:5,
      transformOrigin:"50% 0"
    })
    gsap.to(lines, {
      y:0,
      z:0,
      rotationX:0,
      rotationY:0,
      opacity:1,
      duration,
      stagger:0.05,
      ease:"power2.out",
      scrollTrigger:{
        trigger:$scope,
        start:"top 80%",
        end:"+=100%",
        ...reStart
      }
    })
  },
  CenterFadeInChild($scope){
    var duration = this.defaults.duration;
    var toggle = this.toggleAnimations;
    var _this = this;
    var reStart = toggle ? {toggleActions:"restart none none reverse"} : {};
    var client = _this.client($scope);
    $scope.querySelectorAll(':scope > *').forEach(el=>{
      var position = _this.position(el, $scope);
      var x = (client.width / 2) - position.left;
      var y = (client.height / 2) - position.top;
      gsap.set(el, {x, y, opacity:0});
    })
    gsap.to($scope.querySelectorAll(':scope > *'), {  
      x:0,
      y:0,
      opacity:1,
      duration,
      ease:"back.out(1)", 
      scrollTrigger:{
        trigger:$scope,
        start:"top 80%",
        end:"+=100%",
        ...reStart
      }
    })
  },
  strToHtml(str){
    let parser = new DOMParser();
    return parser.parseFromString(str, "text/html");
  },
  pageTransition(before = ()=>{}, enter = () =>{}, afterEnter = ()=>{}){
    let _this = this;
    let preLogoWrap = document.querySelector(".pt__wrap");
    let preLogo = preLogoWrap.querySelector(".custom-eye");
    let preloadText = preLogoWrap.querySelector(".page-title");
    gsap.set(preLogo, {
      scale:0
    })
    barba.init({
      debug: false,      
      preventRunning:true,     
      timeout: 20000,  
      prevent: ({ el }) =>{},  
      schema: {  
        prefix: 'data-template',
        wrapper: 'wrap',
        container: 'container',
        namespace: 'name',
        limit: 0,
        html: '',  
      },
      transitions:[  
        {
          sync:true,
          name:'global',
          before(data){
            before(data);
          },
          leave(data){
            let done = this.async();
            let nextHTML = _this.strToHtml(data.next.html);
            preloadText.textContent = nextHTML.querySelector('[data-template-name]').getAttribute("data-template-name").replace('-', ' ');
            gsap.timeline()
            .to(preLogo, {
              scale:1,
              transformOrigin:"center bottom",
              onComplete(){
                done();
              },
              ease:"expoScale(0.5,7,none)",
              duration:1,
              onStart(){
                gsap.to(preLogoWrap, {
                  "display":"flex"
                })
              }
            })
            .to(preLogo, { 
              scale:0,
              delay:.1,
              transformOrigin:"center top",
              ease:"expoScale(0.5,7,none)",
              duration:1,
              onStart(){
                enter(data)
                setTimeout(() => {
                  afterEnter(data);
                }, 500);
              },
              onComplete(){
                gsap.to(preLogoWrap, {
                  "display":"none"
                })
              }
            });
          },
          enter(data){
            let nextHTML = _this.strToHtml(data.next.html);
            let nextBodyClass = nextHTML.body.getAttribute('class');
            document.body.setAttribute('class', nextBodyClass ? nextBodyClass : "");
          }
        }
      ]
    })
  },
  slideToggle(content, duration = .5 , type = "toggle"){
    const isHidden = getComputedStyle(content).display === "none";
    if (isHidden) {
        // Slide Down
        content.style.display = "block";
        const fullHeight = content.scrollHeight;
        const computedStyle = window.getComputedStyle(content);
        gsap.fromTo(
            content,
            {
                height: 0,
                paddingTop: 0,
                paddingBottom: 0,
                marginTop: 0,
                marginBottom: 0,
                overflow: "hidden",
            },
            {
                height: fullHeight,
                paddingTop: parseFloat(computedStyle.paddingTop),
                paddingBottom: parseFloat(computedStyle.paddingBottom),
                marginTop: parseFloat(computedStyle.marginTop),
                marginBottom: parseFloat(computedStyle.marginBottom),
                duration,
                ease:"expo.out",
                onComplete: () => {
                    // Clean up styles
                    gsap.set(content, { height: "", paddingTop: "", paddingBottom: "", marginTop: "", marginBottom: "", overflow: "" });
                },
            }
        );
    } else {
        // Slide Up
        gsap.fromTo(
            content,
            {
                height: content.offsetHeight,
                paddingTop: parseFloat(window.getComputedStyle(content).paddingTop),
                paddingBottom: parseFloat(window.getComputedStyle(content).paddingBottom),
                marginTop: parseFloat(window.getComputedStyle(content).marginTop),
                marginBottom: parseFloat(window.getComputedStyle(content).marginBottom),
                overflow: "hidden",
            },
            {
                height: 0,
                paddingTop: 0,
                paddingBottom: 0,
                marginTop: 0,
                marginBottom: 0,
                duration,
                ease:"expo.out",
                onComplete: () => {
                    // Hide and clean styles
                    content.style.display = "none";
                    gsap.set(content, { height: "", paddingTop: "", paddingBottom: "", marginTop: "", marginBottom: "", overflow: "" });
                },
            }
        );
    }
    return isHidden;
  },
  portfolio($scope){  
    var _this = this;
   $scope.querySelectorAll(".portfolio-list-item .btn-toggle").forEach(btn=>{
    btn.addEventListener("click", ()=>{
      var content = btn.closest(".portfolio-list-item").querySelector('.content');
      btn.closest(".portfolio-list-item").classList.toggle("active");
      _this.slideToggle(content);
    });
   });
  },
  caseCard($scope){
    let inner = $scope.querySelector(".card-inner");
    let duration = this.defaults.duration;
    var toggle = this.toggleAnimations;
    var reStart = toggle ? {toggleActions:"restart none none reverse"} : {};
    gsap.set(inner,{
      y:200,
      z:-41,
      opacity:0,
      rotationX:-20,
      transformOrigin:"bottom"
    })
    gsap.to(inner, {
      y:0,
      z:0,
      rotationX:0,
      opacity:1,
      duration,
      ease:"power2.out",
      scrollTrigger:{
        trigger:$scope,
        start:"top 80%",
        end:"+=100%",
        ...reStart
      }
    })
  },
  ContactPageAni($scope){
    // var heroShape = $scope.querySelector(".contact-hero .bg-shape");
    var AniName = $scope.querySelectorAll("[data-ani-name]");
    var AniName2 = $scope.querySelectorAll("[data-ani-name2]");
    var AniSplit = $scope.querySelectorAll(".contact__split-text");
    var checkPreload = $scope.querySelectorAll("[data-check-preload]");
    var beforeAni = this.beforeAni(checkPreload);
    var _this = this;
    AniName2.forEach(el=>_this.cssAni(el));
    setTimeout(() => {
      beforeAni.done();
      AniName.forEach(el=>_this.cssAni(el));
      AniSplit.forEach(el=>_this.splitingLines(el, 'lines'));
    }, 500);
  }
}
/**
 * Start Initialization Object of JrWebDesign
 * */ 
JrWebDesign.smoothScroll();
function initJr(){ 
  document.querySelectorAll(".header__area").forEach(el=>JrWebDesign.header(el));
  document.querySelectorAll(".hero-section").forEach(el=>JrWebDesign.hero(el));
  document.querySelectorAll(".auto__scroll").forEach(el=>JrWebDesign.autoScroll(el));
  document.querySelectorAll(".btn").forEach(el=>JrWebDesign.button(el));
  document.querySelectorAll(".data-ani-stagger").forEach(el=>JrWebDesign.cssAni(el));
  document.querySelectorAll("[data-parent-ani]").forEach(el=>JrWebDesign.parentAni(el));
  document.querySelectorAll(".service__section-pin").forEach(el=>JrWebDesign.Service(el));
  document.querySelectorAll(".process-section").forEach(el=>JrWebDesign.process(el)); 
  document.querySelectorAll(".jrWebDesignSlide").forEach(el=>JrWebDesign.initSlide(el)); // slide
  document.querySelectorAll(".intro-section").forEach(el=>JrWebDesign.introSection(el)); 
  document.querySelectorAll(".teams-wrapper").forEach(el=>JrWebDesign.teamWrap(el));
  document.querySelectorAll(".pined-content").forEach(el=>JrWebDesign.pinedContent(el));
  document.querySelectorAll(".service-cards-y-wrap").forEach(el=>JrWebDesign.serviceCardsY(el));
  document.querySelectorAll(".logo__draw").forEach(el=>JrWebDesign.drawLogo(el));
  document.querySelectorAll(".animated__number").forEach(el=>JrWebDesign.animatedNumber(el));
  document.querySelectorAll(".text__waveIn").forEach(el=>JrWebDesign.textWaveIn(el));
  document.querySelectorAll(".portfolio-list").forEach(el=>JrWebDesign.portfolio(el));
  document.querySelectorAll(".center-fade-in-child").forEach(el=>JrWebDesign.CenterFadeInChild(el));
  JrWebDesign.footer();
  JrWebDesign.init();
}

initJr();

function globalAnimation(){
  let beforeAni = JrWebDesign.beforeAni([".split-lines",".text-split-ani", ".has-cl-ani", ".case__card", "[data-check-preload]"]);
  window.scrollTo(0,0);
  ScrollTrigger.refresh(); 
  return{
    start(){
      beforeAni.done();
      document.querySelectorAll("[data-ani]").forEach(el=>JrWebDesign.cssAni(el));
      document.querySelectorAll(".split-lines").forEach(el=>JrWebDesign.splitingLines(el, 'lines'));
      document.querySelectorAll(".text-split-ani").forEach(el=>JrWebDesign.splitingLines(el, 'words'));
      document.querySelectorAll(".has-cl-ani").forEach(el=>JrWebDesign.clippingMasks(el));
      document.querySelectorAll(".about__hero").forEach(el=>JrWebDesign.aboutHero(el));
      document.querySelectorAll(".case__card").forEach(el=>JrWebDesign.caseCard(el));
      document.querySelectorAll(".contact__page-ani").forEach(el=>JrWebDesign.ContactPageAni(el));
    }
  }
}

let jrAnimation = globalAnimation();

JrWebDesign.preload(document.querySelector('.preload__wrap'), 'assets/file/3s.json', ()=>jrAnimation.start());

window.scrollTo(0,0);
window.addEventListener('load', ()=>{
  setTimeout(() => {
    window.scrollTo(0,0);
    setTimeout(() => {
    ScrollTrigger.refresh();  
    }, 200);
  }, 100); 
});
/**
 * JrWebDesign.pageTransition() utility method designed to handle page transition tasks across the website.
 */
JrWebDesign.pageTransition(function(){
    // Before Transition
    // Clear GSAP tweens and timelines
    gsap.globalTimeline.clear();
  }, function(){
    // Enter Transition
    // Kill all ScrollTriggers
    ScrollTrigger.getAll().forEach(trigger => trigger.kill());
    // Init Methods by initJr() function
    initJr();
    jrAnimation = globalAnimation();
  }, function(){
    // After Transition 
    jrAnimation.start();
});